/* gui.c: Graphics User Interface 
 *
 * Copyright (C) 2000  LinuxDA
 *
 */
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <linux/fb.h>
#include <linux/gui.h>
#include <linux/timer.h>

#include <time.h>

#include "mini_gui.h"
#include <linux/mini_db.h>

#include "stamp.h"
#include "keyboard.h"
#include "sys_menu.h"


static void do_events_loop(void);
static time_t	oldTime;

/*** the out line of our bettery ***/
static short battery_head = 2;
static RECT r_battery = {60,150,90,158};

// enum the days of a week
char days_of_week[][4]={
	"Sun","Mon","Tue","Wed","Thu","Fri","Sat"
};
// Record the current status for clock/date bar
typedef enum {
	SHOW_TIME,SHOW_DATE,SHOW_DAY
}CLOCK_SHOW_MODE;
CLOCK_SHOW_MODE clock_day_mode=SHOW_TIME;
// show the time or date or day information according to variant 'clock_day_mode'
// import : r_flag		--- when switching the mode , we want to refresh the string immediately, isn't it?
//												so, when it's value set to 1, the contents will be refresh immediately
static void set_timer_callback(short r_flag)
{
	char *ts;
	struct tm *dc;
	GraphicsContext gc;
	time_t td;
	char s[80],tail[5];
	RECT r;
//	RECT r1={45,0,100,12};
	U16 bty;
	U8 idx;
	short t;
	
	/**** if the menu bar is shown ***/
	if (IsMenuBarHidden(FrmMain) == 0) return;

	idx = GetIndexFromId(FrmMain,IDD_CLOCK);
	r.left = FrmMain[idx].x;
	r.top = FrmMain[idx].y;
	r.right = r.left + FrmMain[idx].w;r.bottom = r.top + FrmMain[idx].h;

	time(&td);
	
	gc.color = GUI_BLACK;gc.mode = Mode_SRC;gc.fill_index = WhitePattern;

	if ((oldTime != td) || (r_flag == 1)) {
		dc = gmtime(&td);
		
		FillRect(r,&gc);	// clear the old contents
		switch (clock_day_mode)
		{
			case SHOW_TIME:
				t = dc->tm_hour;
				strcpy(tail,"am");
				if (t > 12) {t -=12;strcpy(tail,"pm");}
					else if (t == 12) strcpy(tail,"pm");
				strcpy(s,itoa(t));
				strcat(s,":");
				if (dc->tm_min < 10) strcat(s,"0");
				strcat(s,itoa(dc->tm_min));
				//strcat(s,":");
				//if (dc->tm_sec < 10) strcat(s,"0");
				//strcat(s,itoa(dc->tm_sec));
				strcat(s," ");strcat(s,tail);
			break;
			case SHOW_DATE:
				strcpy(s,itoa(dc->tm_mon+1));strcat(s,"/");
				strcat(s,itoa(dc->tm_mday));strcat(s,"/");
				strcat(s,itoa(dc->tm_year+1900));
			break;
			case SHOW_DAY:
				mktime(dc);
				
				strcpy(s,days_of_week[dc->tm_wday]);
			break;
		}	// end of switch clock_day_mode
		// refresh the contents
		DrawStringInRect(&r,s,&gc,FONT_SYS,X_ALIGN_LEFT|Y_ALIGN_CENTER);
		oldTime = td;

		bty = GetBatteryPower();
		ShowRestPower(bty);

	}

}
/******* Show current power of the battery *******/
static void ShowRestPower(short pwr)
{
	RECT r = r_battery;
	short w;
	GraphicsContext gc;

	r.top += 1;
	w = r_battery.right-r_battery.left-battery_head;
	// Draw the left part
	r.right = r.left + (w*pwr)/100;
	gc.fill_index = BlackPattern;gc.mode = Mode_SRC;
	FillRect(r,&gc);

	// Draw the right part
	r.left = r.right+1;
	r.right = r_battery.right - battery_head-1;
	gc.fill_index = WhitePattern;
	FillRect(r,&gc);

	//Draw a light line
	draw_line(r_battery.left+1,r_battery.top+2,r_battery.right-battery_head-1,r_battery.top+2,0);	
}

/******* only show the out line of battery *******/
static void ShowBatteryOutline(void)
{

	RECT r = r_battery;

	draw_line(r.left,r.top,r.left,r.bottom,1);					//left
	draw_line(r.left,r.top,r.right-battery_head,r.top,1);		//top
	draw_line(r.left,r.bottom,r.right-battery_head,r.bottom,1);	//bottom
	draw_line(r.right-battery_head,r.top,r.right-battery_head,r.bottom,1);	//Right

	// now draw the head of the battery
	draw_line(r.right-battery_head,r.top+battery_head,r.right,r.top+battery_head,1);
	draw_line(r.right,r.top+battery_head,r.right,r.bottom-battery_head,1);
	draw_line(r.right-battery_head,r.bottom-battery_head,r.right,r.bottom-battery_head,1);
}
static void InitSystemPref(void)
{
	InitDate();	/* initialize the date to 1/1/2001 */
	SetAutoOffTime(60);	// auto-off time
}

int main(int argc, char *argv[])
{
	int i;

	init_screen();	/* init the MINI_GUI library */
	
	InitSystemPref();	/** initialize all system preferences ***/
		
	//RegisterChFonts();

	/*** Show the calibration and Welcome page at the 1st time ***/	
	//ShowAbout();
	RunApp("calibration");

	//ShowMainWin();

	/*******************************************
	 *	do the Event demo loop here!
	 *
	 *		Mac Tseng 11/2/2000
	 *******************************************/

	do_events_loop();
	

	close_screen();

	return 0;
}

// run apps as a child routine
static void RunApp(char *app)
{
	int pid;
	pid = vfork();
	if (pid == 0) {
		execlp(app,app,NULL);
		//_exit(0);
	}
		else {
				waitpid(pid,NULL,0);
				//printf("Main win,comes from %s\n",app);
				ShowMainWin();
		}
}

static void draw_icons(int start,DBHANDLE fd,char category)
{
	short totalnumber=0,i,idx,lines;
	SYSDBSTRUCT sysdb_data;
	char cate,secu;

	if (fd==NULL) fd=LDAOpenDB("system",0);
	if (fd==-1) return;
	LDASetDBCategory(fd,category);
	totalnumber=LDAGetRecordNumber(fd);
	lines=(totalnumber-1)/3;
	idx = GetIndexFromId(FrmMain,IDD_SCROLL);
	HideComponent(idx,FrmMain);
	// check if the scrollbar is visible
	if (lines>2) {
		scr_list.max=lines;
		ShowComponent(idx,FrmMain);
	}
	
	for (i=0;i<9;i++){
		idx = GetIndexFromId(FrmMain,i+1);
		HideComponent(idx,FrmMain);
		if ((i+start)<totalnumber){
			LDAGetRecord(fd,&sysdb_data,i+start,&cate,&secu);
			memcpy(icon_bmp[i].img,sysdb_data.app_icon,(sysdb_data.icon_width+7)/8*sysdb_data.icon_height);
			strcpy(icon_bmp[i].caption,sysdb_data.app_name);
			strcpy(app_files[i],sysdb_data.app_file);
			icon_bmp[i].width=sysdb_data.icon_width;
			icon_bmp[i].height=sysdb_data.icon_height;
			FrmMain[idx].text=&icon_bmp[i];
			FrmMain[idx].status=COMPONENT_VISIBLE;
			RefreshComponent(idx,FrmMain);
		}	

	}

	LDACloseDB(fd,0);
	return;
}

static void do_events_loop(void)
{
	U8 exitFlag = 1;
	GUI_MSG msg;
	short idx;
	static short LcdContrast=16;
	
	set_timer_callback(1);	// refresh the clock info. immediately
	while (exitFlag) {
		GetMessage(FrmMain,&msg);
		
		set_timer_callback(0);	// refresh it if changed

		switch (msg.event) {
			case EVENT_CLICK:
				switch (msg.id) {
					case IDD_BITMAP1:					
					case IDD_BITMAP2:					
					case IDD_BITMAP3:					
					case IDD_BITMAP4:					
					case IDD_BITMAP5:					
					case IDD_BITMAP6:					
					case IDD_BITMAP7:					
					case IDD_BITMAP8:					
					case IDD_BITMAP9:
						//RunApp(app_files[msg.id-1]);
						CallApp(app_files[msg.id-1]);
						ShowMainWin();
					break;
					case IDD_SCROLL:
						draw_icons(scr_list.position*3,NULL,category_index[cb_category.selected]);
					break;		
					case IDD_COMBOBOX:
						draw_icons(0,NULL,category_index[cb_category.selected]);
					break;
					case IDD_PREF:
						ShowPreferences();
						ShowMainWin();
					break;
					case IDD_ABOUT:
						ShowAbout();
						ShowMainWin();
					break;
					default:
						//DefaultMessageRoutine(FrmMain,&msg);
						
				}
			break;
			case EVENT_PEN_UP:
				switch (msg.id)
				{
					case IDD_CLOCK:	// switch the clock_day mode
						if (clock_day_mode != SHOW_DAY) clock_day_mode ++; else clock_day_mode = SHOW_TIME;
						set_timer_callback(1);	// refresh info immediately
					break;
					// back light, this functions is disabled now, I only set it to 'Invisible'
					case IDD_BACKLIGHT:
						SwitchBackLight();
					break;
					case IDD_LIGHTER:
						if (LcdContrast <= 40)
						{
							LcdContrast +=4;
							SetContrast(LcdContrast);
						}
					break;
					case IDD_DARKER:
						if (LcdContrast >= 4)
						{
							LcdContrast -=4;
							SetContrast(LcdContrast);
						}
					break;
					default:
						DefaultMessageRoutine(FrmMain,&msg);
				}	// end of EVENT_PENUP -- msg.id
			break;
			case EVENT_INPUTCHAR:
				//printf("%s\n",itoa(msg.keycode));
			break;
			
			case EVENT_KEYUP:
				switch(msg.keycode)
				{
					case KEY_UP:
						if (scr_list.position > 0)
						{
							scr_list.position--;
							idx = GetIndexFromId(FrmMain,IDD_SCROLL);
							RefreshComponent(idx,FrmMain);
							draw_icons(scr_list.position*3,NULL,category_index[cb_category.selected]);
						}
					break;
					case KEY_DOWN:
						if (scr_list.position < scr_list.max)
						{ 
							scr_list.position++;
							idx = GetIndexFromId(FrmMain,IDD_SCROLL);
							RefreshComponent(idx,FrmMain);
							draw_icons(scr_list.position*3,NULL,category_index[cb_category.selected]);
						}
					break;
					case KEY_ADDRESS:
						//printf("address");
						RunApp("address");
					break;
					case KEY_LEFT:	// clock
						RunApp("datebook");
					break;
					case KEY_RIGHT:	// todo
						RunApp("todo");
					break;
					case KEY_CALCULATOR:	// calculator
						RunApp("calculator");
					break;
					case KEY_CLOCK:	//clock
						RunApp("clock");
					break;
				}
			break;
			default:
				DefaultMessageRoutine(FrmMain,&msg);
			
		} /*** end of switch msg.event ****/
	};
}
/*** register the app & regist it to system database  ***/
int	RegisterApp(DBHANDLE db_fd,int icon_width,int icon_height,void * icon,char * app_name,
		char * app_file,char category)
{
	SYSDBSTRUCT sysdb_data;
	int fd;
	
	if (db_fd==0) db_fd=LDAOpenDB("system",0);
	if (db_fd!=-1){
		strcpy(sysdb_data.app_name,app_name);
		strcpy(sysdb_data.app_file,app_file);
//		printf("APP %s\n",app_name);
		//fd=open(app_name,O_RDONLY);
		sysdb_data.app_size=0;//lseek(fd,0,SEEK_END);
//		printf("size: %d,%d,%d\n",icon_width,icon_height,(icon_width+7)/8*icon_height);
		memcpy(sysdb_data.app_icon,icon,(icon_width+7)/8*icon_height);
//		printf("finished\n");
		sysdb_data.icon_width=icon_width;
		sysdb_data.icon_height=icon_height;	

		return LDAAddRecord(db_fd,&sysdb_data,category,0);
	}
}


DBHANDLE OpenSysDB(char * dbname)
{
	DBHANDLE	sysdb_fd;
	DBFIELD db_field[7]={{"AppName",16,DBSTRING},{"AppFile",32,DBSTRING},{"AppSize",sizeof(int),DBINT},{"AppIcon",128,DBBINARY},
			{"AppWidth",sizeof(int),DBINT},{"AppHeight",sizeof(int),DBINT},{"AppDate",sizeof(long),DBDATE}};

	sysdb_fd=LDAOpenDB(dbname,0);
	if (sysdb_fd==-1) {
		if (LDACreateDB(dbname,0,db_field,6)==0){
			sysdb_fd=LDAOpenDB(dbname,0);
			if (sysdb_fd!=-1){
				LDAAddDBCategory(sysdb_fd,"Unfiled");	
				LDAAddDBCategory(sysdb_fd,"Application");	
				LDAAddDBCategory(sysdb_fd,"System");	
				LDAAddDBCategory(sysdb_fd,"Game");	
//				printf("Step category added\n");
//				RegisterApp(sysdb_fd,icon_about_width,icon_about_height,icon_about_bitmap,"About","about",2);
				RegisterApp(sysdb_fd,icon_calculator_width,icon_calculator_height,icon_calculator_bitmap,"prog1","prog1",1);
//				RegisterApp(sysdb_fd,icon_todo_width,icon_todo_height,icon_todo_bitmap,"To do","todo",1);
				RegisterApp(sysdb_fd,icon_mine_width,icon_mine_height,icon_mine_bitmap,"prog2","prog2",3);
				RegisterApp(sysdb_fd,icon_mine_width,icon_mine_height,icon_mine_bitmap,"prog3","prog3",3);
//				RegisterApp(sysdb_fd,icon_browser_width,icon_browser_height,icon_browser_bitmap,"Browser",1);
				RegisterApp(sysdb_fd,icon_paint_width,icon_paint_height,icon_paint_bitmap,"prog4","prog4",1);
				RegisterApp(sysdb_fd,icon_calibration_width,icon_calibration_height,icon_calibration_bitmap,"prog5","prog5",2);
				RegisterApp(sysdb_fd,icon_memopad_width,icon_memopad_height,icon_memopad_bitmap,"prog6","prog6",1);
				RegisterApp(sysdb_fd,icon_address_width,icon_address_height,icon_address_bitmap,"prog7","prog7",1);
				RegisterApp(sysdb_fd,icon_hardball_width,icon_hardball_height,icon_hardball_bitmap,"prog8","prog8",3);
				RegisterApp(sysdb_fd,icon_expense_width,icon_expense_height,icon_expense_bitmap,"prog9","prog9",1);
				
				return sysdb_fd;
			}
		} else{
			return -1;
		}
	} else return sysdb_fd;
}
// system reboot, need to initialize the date
void InitDate(void)
{
	time_t td;
	long a;
 	
 	struct tm now;
	time(&td);
	//printf("t\n%d,\n",td);

	now.tm_hour=0;
	now.tm_min=0;
	now.tm_sec=0;
	now.tm_year=101;	/*** linux date start at year 1900 **/
	now.tm_mon=0;
	now.tm_mday=1;		/*** 0 to 30 ***/
	
	td=mktime(&now);
	
	
	stime(&td);
	

}
// necessary to hide all the icons so that invoid the ugly duplicate refresh
void HideAllIcons(void)
{
	short idx;
	idx = GetIndexFromId(FrmMain,IDD_BITMAP1);
	FrmMain[idx].status = COMPONENT_INVISIBLE;

	idx = GetIndexFromId(FrmMain,IDD_BITMAP2);
	FrmMain[idx].status = COMPONENT_INVISIBLE;

	idx = GetIndexFromId(FrmMain,IDD_BITMAP3);
	FrmMain[idx].status = COMPONENT_INVISIBLE;

	idx = GetIndexFromId(FrmMain,IDD_BITMAP4);
	FrmMain[idx].status = COMPONENT_INVISIBLE;

	idx = GetIndexFromId(FrmMain,IDD_BITMAP5);
	FrmMain[idx].status = COMPONENT_INVISIBLE;

	idx = GetIndexFromId(FrmMain,IDD_BITMAP6);
	FrmMain[idx].status = COMPONENT_INVISIBLE;

	idx = GetIndexFromId(FrmMain,IDD_BITMAP7);
	FrmMain[idx].status = COMPONENT_INVISIBLE;

	idx = GetIndexFromId(FrmMain,IDD_BITMAP8);
	FrmMain[idx].status = COMPONENT_INVISIBLE;

	idx = GetIndexFromId(FrmMain,IDD_BITMAP9);
	FrmMain[idx].status = COMPONENT_INVISIBLE;
}
void ShowMainWin(void)
{
	RECT r;
	GraphicsContext gc;
	DBHANDLE fd;
	short totalnumber=0,i,idx;

	//display_bitmap(elephant_bitmap);
	fd=OpenSysDB("system");
	totalnumber=LDAGetDBCategoryTotal(fd);
	
	category_index[0]=DB_CATEGORY_ALL;
	strcpy(cb_items[0].str,"All");
	cb_items[0].index=0;
	for (i=1;i<=totalnumber;i++){
		category_index[i]=LDAGetDBCategoryAlphbet(fd,i-1,cb_items[i].str);
		cb_items[i].index=i;
	}
	idx = GetIndexFromId(FrmMain,IDD_COMBOBOX);
	RefreshComponent(idx,FrmMain);

	HideAllIcons();	// necessary to hide all the icons so that invoid the ugly duplicate refresh
	CreateForm(FrmMain);

	
	draw_icons(scr_list.position*3,fd,category_index[cb_category.selected]);
	

	//Draw bottom Line
	draw_line(0,148,159,148,1);					//left
	
	// battery
	ShowBatteryOutline();
	return;
}
